# -*- coding: utf-8 -*-

import os

import click
from werkzeug.serving import run_simple

import revelation
from revelation.utils import (download_reveal, extract_file, make_presentation,
                              move_and_replace)


@click.group(invoke_without_command=True)
@click.option('--version', '-v', is_flag=True, default=False)
@click.pass_context
def cli(ctx, version):
    if not ctx.invoked_subcommand and version:
        click.echo(revelation.__version__)
        ctx.exit()
    elif not ctx.invoked_subcommand:
        click.echo(ctx.get_help())


@cli.command('installreveal', help='Install or upgrade reveal.js dependency')
@click.option('--url', '-u', help='Reveal.js download url')
@click.pass_context
def installreveal(ctx, url):
    click.echo('Downloading reveal.js...')

    download = download_reveal(url)

    click.echo('Installing reveal.js...')

    move_and_replace(
        extract_file(download[0]),
        os.path.join(os.path.dirname(revelation.__file__), 'static')
    )

    click.echo('Installation completed!')


@cli.command('mkpresentation', help='Create a new revelation presentation')
@click.argument('presentation')
@click.pass_context
def mkpresentation(ctx, presentation):
    if os.path.exists(presentation):
        click.echo('This presentation already exists')
        ctx.exit()

    click.echo('Starting a new presentation...')
    make_presentation(presentation)


@cli.command('start', help='Start the revelation server')
@click.argument('presentation', default=os.getcwd())
@click.option('--port', '-p', default=4000, help='Presentation server port')
@click.option('--config', '-c', default=None, help='Custom configuration file')
@click.option('--media', '-m', default=None, help='Custom media folder')
@click.option('--debug', '-d', is_flag=True, default=False,
              help='Run the revelation server in debugger and reloader on')
@click.pass_context
def start(ctx, presentation, port, config, media, debug):
    # Check if reveal.js is installed
    if not os.path.exists(os.path.join(
            os.path.dirname(revelation.__file__), 'static')):
        click.echo('You must run installreveal command first')
        ctx.exit()

    # Check for presentation file
    if os.path.isfile(presentation):
        path = os.path.dirname(presentation)
    else:
        click.echo('Presentation file not found')
        ctx.exit()

    # Check for media root
    if not media:
        media = os.path.join(path, 'media')

    if not os.path.isdir(media):
        # Running without media folder
        media = None

        click.echo('Media folder not detected, running without media')

    # Check for configuration file
    if not config:
        config = os.path.join(path, 'config.py')

    if not os.path.isfile(config):
        # Running without configuration file
        config = None

        click.echo('Configuration file not detected, running with defaults')

    click.echo('Starting revelation server...')

    # instatiating revelatino app
    app = revelation.Revelation(presentation, media, config)

    if debug:
        run_simple(
            'localhost', port, app, use_debugger=True, use_reloader=True)
    else:
        run_simple('localhost', port, app)



# -*- coding: utf-8 -*-

REVEAL_META = {
    # Title of the slide
    'title': 'The title',

    # Author in the metadata of the slide
    'author': 'Some Author',

    # Description in the metadata of the slide
    'description': 'Some description'
}

# Reveal markdown slide separator
REVEAL_SLIDE_SEPARATOR = '---'

# Themes
# beige, black, blood, league, moon, night, serif, simple, sky,
# solarized, white
REVEAL_THEME = 'black'

REVEAL_CONFIG = {
    # Display controls in the bottom right corner
    'controls': True,

    # Display a presentation progress bar
    'progress': True,

    # Display the page number of the current slide
    'slideNumber': False,

    # Push each slide change to the browser history
    'history': True,

    # Enable keyboard shortcuts for navigation
    'keyboard': True,

    # Enable the slide overview mode
    'overview': True,

    # Vertical centering of slides
    'center': True,

    # Enables touch navigation on devices with touch input
    'touch': True,

    # Loop the presentation
    'loop': False,

    # Change the presentation direction to be RTL
    'rtl': False,

    # Turns fragments on and off globally
    'fragments': True,

    # Flags if the presentation is running in an embedded mode,
    # i.e. contained within a limited portion of the screen
    'embedded': False,

    # Flags if we should show a help overlay when the questionmark
    # key is pressed
    'help': True,

    # Flags if speaker notes should be visible to all viewers
    'showNotes': False,

    # Number of milliseconds between automatically proceeding to the
    # next slide, disabled when set to 0, this value can be overwritten
    # by using a data-autoslide attribute on your slides
    'autoSlide': 0,

    # Stop auto-sliding after user input
    'autoSlideStoppable': True,

    # Enable slide navigation via mouse wheel
    'mouseWheel': False,

    # Hides the address bar on mobile devices
    'hideAddressBar': True,

    # Opens links in an iframe preview overlay
    'previewLinks': False,

    # Transition style
    # default/cube/page/concave/zoom/linear/fade/none
    'transition': 'default',

    # Transition speed
    'transitionSpeed': 'default',  # default/fast/slow

    # Transition style for full page slide backgrounds
    # default/none/slide/concave/convex/zoom
    'backgroundTransition': 'default',

    # Number of slides away from the current that are visible
    'viewDistance': 3,

    # Parallax background image
    # e.g.:
    # "'https://s3.amazonaws.com/hakim-static/reveal-js/reveal-parallax-1.jpg'"
    'parallaxBackgroundImage': '',

    # Parallax background size
    'parallaxBackgroundSize': '',  # CSS syntax, e.g. "2100px 900px"

    # Amount to move parallax background (horizontal and vertical)
    # on slide change
    # Number, e.g. 100
    'parallaxBackgroundHorizontal': '',
    'parallaxBackgroundVertical': '',
}



# -*- coding: utf-8 -*-

import os
import types

from werkzeug.utils import import_string

from . import default_config


class Config(dict):
    '''
    Class that stores revelation configs thanks to flask Config:

    https://github.com/pallets/flask/blob/master/flask/config.py

    It loads the default_config variables and if a path is passed it also
    loads the external configs
    '''

    def __init__(self, filename=None):
        self.load_from_object(default_config)

        if filename and os.path.isfile(filename):
            self.load_from_pyfile(filename)

    def load_from_object(self, obj):
        if isinstance(obj, str):
            obj = import_string(obj)

        for key in dir(obj):
            if key.isupper():
                self[key] = getattr(obj, key)

    def load_from_pyfile(self, filename):
        d = types.ModuleType('config')
        d.__file__ = filename

        try:
            with open(filename, mode='rb') as config_file:
                exec(compile(config_file.read(), filename, 'exec'), d.__dict__)
        except IOError as e:
            e.strerror = 'Unable to load configuration file (%s)' % e.strerror
            raise

        self.load_from_object(d)



# -*- coding: utf-8 -*-

import os
import re

from jinja2 import Environment, PackageLoader, select_autoescape
from werkzeug.wrappers import Request, Response
from werkzeug.wsgi import SharedDataMiddleware

from .config import Config


class Revelation(object):
    '''
    Main revelation app class that instantiates the server and handles
    the requests
    '''

    def __init__(self, presentation, media=None, config=None):
        '''
        Initializes the server and creates the environment for the presentation
        '''
        self.config = Config(config)
        self.presentation = presentation

        shared_data = {
            '/static': os.path.join(os.path.dirname(__file__), 'static')
        }

        shared_data.update(self.parse_media_root(media))

        self.wsgi_app = SharedDataMiddleware(self.wsgi_app, shared_data)

    def parse_media_root(self, media_root):
        '''
        Create the shared_data configuration for media_root
        if the folder exists
        '''
        if media_root:
            media_root = os.path.abspath(media_root)

            if os.path.isdir(media_root):
                media_url = '/{}'.format(os.path.basename(media_root))

                return {media_url: media_root}

        return {}

    def load_slides(self, path, separator):
        '''
        Get slides file from the given path, loads it and split into list
        of slides.

        :return: a list of strings with the slides content
        '''
        with open(path, 'r') as presentation:
            slides = presentation.read()

        return re.split('^{}$'.format(separator), slides, flags=re.MULTILINE)

    def dispatch_request(self, request):
        env = Environment(
            loader=PackageLoader('revelation', 'templates'),
            autoescape=select_autoescape(['html'])
        )

        context = {
            'meta': self.config.get('REVEAL_META'),
            'slides': self.load_slides(
                self.presentation, self.config.get('REVEAL_SLIDE_SEPARATOR')),
            'config': self.config.get('REVEAL_CONFIG'),
            'theme': 'static/css/theme/{}.css'.format(
                self.config.get('REVEAL_THEME')),
        }

        template = env.get_template('presentation.html')

        return Response(
            template.render(**context),
            headers={'content-type': 'text/html'}
        )

    def wsgi_app(self, environ, start_response):
        request = Request(environ)
        response = self.dispatch_request(request)
        return response(environ, start_response)

    def __call__(self, environ, start_response):
        return self.wsgi_app(environ, start_response)



# -*- coding: utf-8 -*-

import os
import shutil
import tarfile
import zipfile

try:
    # python 3
    from urllib.request import urlretrieve
except ImportError:
    # python 2
    from urllib import urlretrieve

from . import default_config

REVEAL_URL = 'https://github.com/hakimel/reveal.js/archive/3.6.0.tar.gz'


def make_presentation(presentation_path):
    '''
    Make a new presentation boilerplate code given a presentation_path
    '''
    name = os.path.basename(presentation_path)
    # Presentation dir
    os.mkdir(name)
    # Media dir
    os.mkdir(os.path.join(presentation_path, 'media'))
    # Config file
    config_file = os.path.join(
        os.path.dirname(default_config.__file__), 'default_config.py')
    shutil.copy(config_file, os.path.join(presentation_path, 'config.py'))
    # Slide file
    with open(os.path.join(presentation_path, 'slides.md'), 'w') as f:
        f.write('# {0}\n\nStart from here!'.format(
            name.replace('_', ' ').replace('-', ' ').title()))


def download_reveal(url=None):
    '''
    Download reveal.js installation files
    '''
    if not url:
        url = REVEAL_URL

    try:
        return urlretrieve(url)
    except Exception:
        raise


def move_and_replace(src, dst):
    '''
    Helper function used to move files from one place to another,
    creating os replacing them if needed

    :param src: source directory
    :param dst: destination directory
    '''

    src = os.path.abspath(src)
    dst = os.path.abspath(dst)

    for src_dir, _, files in os.walk(src):
        # using os walk to navigate through the directory tree
        # keep te dir structure by replacing the source root to
        # the destination on walked path
        dst_dir = src_dir.replace(src, dst)

        if not os.path.exists(dst_dir):
            os.mkdir(dst_dir)  # to copy not fail, create the not existing dirs

        for file_ in files:
            src_file = os.path.join(src_dir, file_)
            dst_file = os.path.join(dst_dir, file_)

            if os.path.exists(dst_file):
                os.remove(dst_file)  # to copy not fail, create existing files

            shutil.move(src_file, dst_dir)  # move the files

    shutil.rmtree(src)  # remove the dir structure from the source


def extract_file(compressed_file, path='.'):
    if os.path.isfile(compressed_file):
        if tarfile.is_tarfile(compressed_file):
            with tarfile.open(compressed_file, 'r:gz') as tfile:
                basename = tfile.members[0].name
                tfile.extractall(path+'/')
        elif zipfile.is_zipfile(compressed_file):
            with zipfile.ZipFile(compressed_file, 'r') as zfile:
                basename = zfile.namelist()[0]
                zfile.extractall(path)

    return os.path.abspath(os.path.join(path, basename))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''revelation setup file'''

import os
import re

from setuptools import find_packages, setup

try:
    import pypandoc
    README = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError):
    README = ''

# package variables
PACKAGE = 'revelation'
REQUIREMENTS = [
    'Jinja2==2.10',
    'Werkzeug==0.14.1',
    'click==6.7',
]
TEST_REQUIREMENTS = [
    'coverage==4.4.1',
    'mock==2.0.0',
    'nose==1.3.7',
    'pylint==1.7.1',
]

# dynamic package info
with open(os.path.join(PACKAGE, '__init__.py')) as init_file:
    INIT = init_file.read()

VERSION = re.search(
    "^__version__ = ['\"]([^'\"]+)['\"]", INIT, re.MULTILINE).group(1)
AUTHOR = re.search(
    "^__author__ = ['\"]([^'\"]+)['\"]", INIT, re.MULTILINE).group(1)
EMAIL = re.search(
    "^__email__ = ['\"]([^'\"]+)['\"]", INIT, re.MULTILINE).group(1)

setup(
    name=PACKAGE,
    version=VERSION,
    description='Make awesome reveal.js presentations with revelation',
    long_description=README,
    author=AUTHOR,
    author_email=EMAIL,
    url='https://github.com/humrochagf/revelation',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIREMENTS,
    entry_points=dict(
        console_scripts=['revelation=revelation.cli:cli']),
    platforms='any',
    keywords=['presentation', 'slides', 'reveal.js', 'markdown'],
    classifiers=[
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Multimedia :: Graphics :: Presentation',
        'Topic :: Text Processing :: Markup :: HTML',
    ],
    test_suite='tests',
    tests_require=TEST_REQUIREMENTS
)
